/************************************************************************/
/*   Two board example using EM2 to make the connection for data only   */
/*   This is board0.c - for the first board                             */
/*   Use the make new project plug in to make a Server/Loader project   */
/*   using this file, but for the module type that you have.            */
/*   That project must be modified to have the "readtask" task, and     */
/*   also the "read0" and "write0" semaphores.                          */
/*   Refer to the pdf document for full details                         */
/*												*/
/*   version 	Author	date		reason				*/
/*   -------      ------      ----		------				*/
/*   1.0          PW         	28/11/02	first written			*/
/*												*/
/************************************************************************/ 


#include <stdioc60.h>
#include <std.h>
#include <swi.h>
#include <tsk.h>
#include <sem.h>


#define FIFONO 1       /* the FIFO that is routed to the EM2 in the network file*/
#define BLOCKSIZE 500  /*whatever you want but MUST be the same as the other board*/

#include "heron4.h"

#pragma FUNC_EXT_CALLED(maintask)

extern far SEM_Obj read0;  
extern far SEM_Obj write0; 



int main(void)
{
   /* For SL, do a call to bootloader first!*/
   bootloader();
}

/* WARNING the default stack size for this task ("maintask") defined in  */
/* the cdb file is 8192 bytes.                                          */
/* This means that you cannot statically declare more than 2048 integers */
/* in this task.                                                         */
/* You can increase the stack size allowed by changing the settings for  */
/* the task in the cdb file.                                             */
/* If you need more than 2048 integers, it is better to use Malloc/Free  */
/* to allocate your data buffers                                         */

int maintask(void)
{

	HeronFIFO *wfifo;
	int wstatus;
	int *databuf;
	int i;

	/*
	 * All processors in the system are required to clear the config line
	 * to an in-active state. One all processors in a system performed a
	 * HeronConfigOff(), the config line goes in-active, and GDIO modules
	 * are only then able to send data onto C6xxx modules. 
	 * If you only use processor modules but no GDIO modules, then this
	 * doesn't matter to you and you can leave the config line in any
	 * state (active -> don't do a HeronConfigOff() or in-active -> do a
	 * HeronConfigOff() 
	 */
	HeronConfigOff();

	printf("Bd0: HERON module write task has started running\n");

	databuf=(int *)malloc(BLOCKSIZE*sizeof(int));

	if (databuf==NULL) 
	{
		printf("Bd0: error with write malloc\n");
		exit(0);
	}
	

	/* open the FIFO for writing*/
    	wfifo=HeronSemOpenFifo(FIFONO,"w",&write0);
    	if (wfifo==NULL)
     	{
     		printf("Bd0: Cannot open fifo %d for write. Error %d\n",FIFONO,heronerr);
     		exit(-1); 	
     	} 	
	
	
while(1) /* send data blocks*/
{

	for (i=0; i<BLOCKSIZE; i++)
	{
	databuf[i]=i;
	}

	/* send the pattern to the other module */
 	wstatus = HeronWrite(wfifo,databuf,BLOCKSIZE);
	if (wstatus != HERON_IO_IN_PROGRESS)
	{
	printf("Bd0: Cannot write fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}

	TSK_sleep(1); /* to make sure that the read task gets scheduled sometimes*/
    	SEM_pend(&write0,SYS_FOREVER);    
}    
    

}/* end of maintask*/

    
    
int readtask(void) 
{

	HeronFIFO *rfifo;
	int statusr;
	int *results;
	int i;
	int errors=0;
	int blockcount=0;

	printf("Bd0: HERON module read task has started running\n");

	results=(int *)malloc(500*sizeof(int));

	if (results==NULL) 
	{
		printf("Bd0: error with read malloc\n");
		exit(0);
	}
	

	/* open the FIFO for reading*/
    	rfifo=HeronSemOpenFifo(FIFONO,"r",&read0);
    	if (rfifo==NULL)
     	{
     		printf("Bd0: Cannot open fifo %d for read. Error %d\n",FIFONO,heronerr);
     		exit(-1); 	
     	}	 	
	
	
while(1) /* read blocks and check them */
{
	errors = 0;
	for (i=0; i<BLOCKSIZE; i++)
	{
	results[i]=0xc0ffee;
	}

	statusr = HeronRead(rfifo,results,BLOCKSIZE);

	if (statusr != HERON_IO_IN_PROGRESS)
	{
	printf("Bd0: Cannot read fifo %d. Error %d\n",FIFONO,heronerr);
	exit(-1);
	}
	TSK_sleep(1); /* to make sure that the maintask gets run*/
   	SEM_pend(&read0,SYS_FOREVER); 	
	/* check the data*/
	for (i=0;i<BLOCKSIZE;i++)
	{
		if (results[i] != i) 
		{
			errors++;
			printf("Bd0: dedicated :error received %x not %x\n",results[i],i);
		}
	}

	
	if (errors)
	{
		printf("Bd0: %d Errors in data block test. Hit enter to continue\n", errors);
		getchar();
	}
blockcount++;
if((blockcount%100)==0)printf("board0 completed loop %d\n",blockcount++);    
  
  
}    
  
    
    
} /* end of readtask*/  
   
